import unittest

import openmdao.api as om
from openmdao.utils.assert_utils import assert_check_partials, assert_near_equal

from aviary.subsystems.aerodynamics.gasp_based.flaps_model.flaps_model import \
    FlapsGroup
from aviary.variable_info.options import get_option_defaults
from aviary.variable_info.enums import FlapType
from aviary.variable_info.variables import Aircraft, Dynamic

"""
All data is from validation files using standalone flaps model
"""


class FlapsGroupTestCaseTripleSlotted(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.FLAP_TYPE,
                        val=FlapType.TRIPLE_SLOTTED, units='unitless')

        self.prob.model = FCC = FlapsGroup(aviary_options=options)

        self.prob.setup()

        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Dynamic.Mission.TEMPERATURE, 518.67, units="degR")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.406626, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

        self.prob.set_val("VLAM1", 0.97217)
        self.prob.set_val("VLAM2", 1.09948)
        self.prob.set_val("VLAM3", 0.97217)
        self.prob.set_val("VLAM4", 1.25725)
        self.prob.set_val("VLAM5", 1.0000)
        self.prob.set_val("VLAM6", 1.0000)
        self.prob.set_val("VLAM7", 0.735)
        self.prob.set_val("VLAM8", 0.74444322)
        self.prob.set_val("VLAM9", 0.9975)
        self.prob.set_val("VLAM10", 0.74)
        self.prob.set_val("VLAM11", 0.84232)
        self.prob.set_val("VLAM12", 0.79208)
        self.prob.set_val("VLAM13", 1.03512)
        self.prob.set_val("VLAM14", 0.99124)

        self.prob.set_val("VDEL1", 1.0)
        self.prob.set_val("VDEL2", 0.62455)
        self.prob.set_val("VDEL3", 0.765)
        self.prob.set_val("VDEL4", 0.93578)
        self.prob.set_val("VDEL5", 0.90761)

        self.prob.set_val(Dynamic.Mission.SPEED_OF_SOUND, 1118.21948771, units="ft/s")
        self.prob.set_val(Aircraft.Wing.LOADING, 128.0, units="lbf/ft**2")
        self.prob.set_val(Dynamic.Mission.STATIC_PRESSURE,
                          (14.696 * 144), units="lbf/ft**2")
        self.prob.set_val(Aircraft.Wing.AVERAGE_CHORD, 12.61, units="ft")
        self.prob.set_val("kinematic_viscosity", 0.15723e-3, units="ft**2/s")
        self.prob.set_val(Aircraft.Wing.MAX_LIFT_REF, 1.150)
        self.prob.set_val(Aircraft.Wing.SLAT_LIFT_INCREMENT_OPTIMUM, 0.930)
        self.prob.set_val("fus_lift", 0.05498)
        self.prob.set_val(Aircraft.Wing.FLAP_LIFT_INCREMENT_OPTIMUM, 1.500)
        self.prob.set_val(Aircraft.Wing.FLAP_DRAG_INCREMENT_OPTIMUM, 0.1)

        self.prob.set_val("flap_defl_ratio", 0.72727273)
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_UNWEIGHTED, 0.13966)
        self.prob.set_val("flap_defl", 40.0, units="deg")
        self.prob.set_val(Aircraft.Wing.FLAP_SPAN_RATIO, 0.65)

        self.prob.set_val("slat_defl_ratio", 0.5)
        self.prob.set_val("body_to_span_ratio", 0.09240447)
        self.prob.set_val("chord_to_body_ratio", 0.12679)
        self.prob.set_val(Aircraft.Wing.SLAT_SPAN_RATIO, 0.89759553)

    def test_case(self):

        self.prob.run_model()
        tol = 6e-4  # checked. high tol for lack of precision in GASP data.

        reg_data = 2.8155
        ans = self.prob["CL_max"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.17522
        ans = self.prob[Dynamic.Mission.MACH]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 157.1111
        ans = self.prob["reynolds"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.0406
        ans = self.prob["delta_CD"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 1.0293
        ans = self.prob["delta_CL"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(method="fd", out_stream=None)
        assert_check_partials(
            data, atol=6315, rtol=7e-3
        )  # tolerance at this value for special cases. All other values check out.


class FlapsGroupTestCaseSplit(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.FLAP_TYPE, val=FlapType.SPLIT, units='unitless')

        self.prob.model = FCC = FlapsGroup(aviary_options=options)

        self.prob.setup()

        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Dynamic.Mission.TEMPERATURE, 518.67, units="degR")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.406626, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

        self.prob.set_val("VLAM1", 0.97217)
        self.prob.set_val("VLAM2", 1.09948)
        self.prob.set_val("VLAM3", 0.97217)
        self.prob.set_val("VLAM4", 1.19742)
        self.prob.set_val("VLAM5", 1.0000)
        self.prob.set_val("VLAM6", 0.8)
        self.prob.set_val("VLAM7", 0.735)
        self.prob.set_val("VLAM8", 0.74444322)
        self.prob.set_val("VLAM9", 0.9975)
        self.prob.set_val("VLAM10", 0.74)
        self.prob.set_val("VLAM11", 0.84232)
        self.prob.set_val("VLAM12", 0.79208)
        self.prob.set_val("VLAM13", 1.03209)
        self.prob.set_val("VLAM14", 0.99082)

        self.prob.set_val("VDEL1", 1.0)
        self.prob.set_val("VDEL2", 0.55667)
        self.prob.set_val("VDEL3", 0.765)
        self.prob.set_val("VDEL4", 0.93578)
        self.prob.set_val("VDEL5", 0.90761)

        self.prob.set_val(Dynamic.Mission.SPEED_OF_SOUND, 1118.21948771, units="ft/s")
        self.prob.set_val(Aircraft.Wing.LOADING, 128.0, units="lbf/ft**2")
        self.prob.set_val(Dynamic.Mission.STATIC_PRESSURE,
                          (14.696 * 144), units="lbf/ft**2")
        self.prob.set_val(Aircraft.Wing.AVERAGE_CHORD, 12.61, units="ft")
        self.prob.set_val("kinematic_viscosity", 0.15723e-3, units="ft**2/s")
        self.prob.set_val(Aircraft.Wing.MAX_LIFT_REF, 1.150)
        self.prob.set_val(Aircraft.Wing.SLAT_LIFT_INCREMENT_OPTIMUM, 0.930)
        self.prob.set_val("fus_lift", 0.05498)
        self.prob.set_val(Aircraft.Wing.FLAP_LIFT_INCREMENT_OPTIMUM, 1.500)
        self.prob.set_val(Aircraft.Wing.FLAP_DRAG_INCREMENT_OPTIMUM, 0.1)

        self.prob.set_val("flap_defl_ratio", 0.6666667)
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_UNWEIGHTED, 0.13966)
        self.prob.set_val("flap_defl", 40.0, units="deg")
        self.prob.set_val(Aircraft.Wing.FLAP_SPAN_RATIO, 0.65)

        self.prob.set_val("slat_defl_ratio", 0.5)
        self.prob.set_val("body_to_span_ratio", 0.09240447)
        self.prob.set_val("chord_to_body_ratio", 0.12679)
        self.prob.set_val(Aircraft.Wing.SLAT_SPAN_RATIO, 0.89759553)

    def test_case(self):

        self.prob.run_model()
        tol = 9e-4  # checked. high tol for lack of precision in GASP data.

        reg_data = 2.56197
        ans = self.prob["CL_max"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.18368
        ans = self.prob[Dynamic.Mission.MACH]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 164.78406
        ans = self.prob["reynolds"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.0362
        ans = self.prob["delta_CD"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.7816
        ans = self.prob["delta_CL"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(method="fd", out_stream=None)
        assert_check_partials(
            data, atol=6630, rtol=0.007
        )  # tolerance at this value for special cases. All other values check out.


class FlapsGroupTestCaseSingleSlotted(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.FLAP_TYPE,
                        val=FlapType.SINGLE_SLOTTED, units='unitless')

        self.prob.model = FCC = FlapsGroup(aviary_options=options)

        self.prob.setup()

        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Dynamic.Mission.TEMPERATURE, 518.67, units="degR")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_FLAP_DEFLECTION, 55.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.406626, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

        self.prob.set_val("VLAM1", 0.97217)
        self.prob.set_val("VLAM2", 1.09948)
        self.prob.set_val("VLAM3", 0.97217)
        self.prob.set_val("VLAM4", 1.25725)
        self.prob.set_val("VLAM5", 1.0000)
        self.prob.set_val("VLAM6", 1.0000)
        self.prob.set_val("VLAM7", 0.735)
        self.prob.set_val("VLAM8", 0.74444322)
        self.prob.set_val("VLAM9", 0.9975)
        self.prob.set_val("VLAM10", 0.74)
        self.prob.set_val("VLAM11", 0.84232)
        self.prob.set_val("VLAM12", 0.79208)
        self.prob.set_val("VLAM13", 1.03512)
        self.prob.set_val("VLAM14", 0.99124)

        self.prob.set_val("VDEL1", 1.0)
        self.prob.set_val("VDEL2", 0.62455)
        self.prob.set_val("VDEL3", 0.765)
        self.prob.set_val("VDEL4", 0.93578)
        self.prob.set_val("VDEL5", 0.90761)

        self.prob.set_val(Dynamic.Mission.SPEED_OF_SOUND, 1118.21948771, units="ft/s")
        self.prob.set_val(Aircraft.Wing.LOADING, 128.0, units="lbf/ft**2")
        self.prob.set_val(Dynamic.Mission.STATIC_PRESSURE,
                          (14.696 * 144), units="lbf/ft**2")
        self.prob.set_val(Aircraft.Wing.AVERAGE_CHORD, 12.61, units="ft")
        self.prob.set_val("kinematic_viscosity", 0.15723e-3, units="ft**2/s")
        self.prob.set_val(Aircraft.Wing.MAX_LIFT_REF, 1.150)
        self.prob.set_val(Aircraft.Wing.SLAT_LIFT_INCREMENT_OPTIMUM, 0.930)
        self.prob.set_val("fus_lift", 0.05498)
        self.prob.set_val(Aircraft.Wing.FLAP_LIFT_INCREMENT_OPTIMUM, 1.500)
        self.prob.set_val(Aircraft.Wing.FLAP_DRAG_INCREMENT_OPTIMUM, 0.1)

        self.prob.set_val("flap_defl_ratio", 0.72727273)
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_UNWEIGHTED, 0.13966)
        self.prob.set_val("flap_defl", 40.0, units="deg")
        self.prob.set_val(Aircraft.Wing.FLAP_SPAN_RATIO, 0.65)

        self.prob.set_val("slat_defl_ratio", 0.5)
        self.prob.set_val("body_to_span_ratio", 0.09240447)
        self.prob.set_val("chord_to_body_ratio", 0.12679)
        self.prob.set_val(Aircraft.Wing.SLAT_SPAN_RATIO, 0.89759553)

    def test_case(self):

        self.prob.run_model()
        tol = 6e-4  # checked. high tol for lack of precision in GASP data.

        reg_data = 2.8155
        ans = self.prob["CL_max"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.17522
        ans = self.prob[Dynamic.Mission.MACH]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 157.1111
        ans = self.prob["reynolds"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.0406
        ans = self.prob["delta_CD"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 1.0293
        ans = self.prob["delta_CL"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(method="fd", out_stream=None)
        assert_check_partials(
            data, atol=6315, rtol=0.007
        )  # tolerance at this value for special cases. All other values check out.


class FlapsGroupTestCasePlain(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.FLAP_TYPE, val=FlapType.PLAIN, units='unitless')

        self.prob.model = FCC = FlapsGroup(aviary_options=options)

        self.prob.setup()

        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Dynamic.Mission.TEMPERATURE, 518.67, units="degR")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.406626, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

        self.prob.set_val("VLAM1", 0.97217)
        self.prob.set_val("VLAM2", 1.09948)
        self.prob.set_val("VLAM3", 0.97217)
        self.prob.set_val("VLAM4", 1.19742)
        self.prob.set_val("VLAM5", 1.0000)
        self.prob.set_val("VLAM6", 0.8)
        self.prob.set_val("VLAM7", 0.735)
        self.prob.set_val("VLAM8", 0.74444322)
        self.prob.set_val("VLAM9", 0.9975)
        self.prob.set_val("VLAM10", 0.74)
        self.prob.set_val("VLAM11", 0.84232)
        self.prob.set_val("VLAM12", 0.79208)
        self.prob.set_val("VLAM13", 1.03209)
        self.prob.set_val("VLAM14", 0.99082)

        self.prob.set_val("VDEL1", 1.0)
        self.prob.set_val("VDEL2", 0.55667)
        self.prob.set_val("VDEL3", 0.765)
        self.prob.set_val("VDEL4", 0.93578)
        self.prob.set_val("VDEL5", 0.90761)

        self.prob.set_val(Dynamic.Mission.SPEED_OF_SOUND, 1118.21948771, units="ft/s")
        self.prob.set_val(Aircraft.Wing.LOADING, 128.0, units="lbf/ft**2")
        self.prob.set_val(Dynamic.Mission.STATIC_PRESSURE,
                          (14.696 * 144), units="lbf/ft**2")
        self.prob.set_val(Aircraft.Wing.AVERAGE_CHORD, 12.61, units="ft")
        self.prob.set_val("kinematic_viscosity", 0.15723e-3, units="ft**2/s")
        self.prob.set_val(Aircraft.Wing.MAX_LIFT_REF, 1.150)
        self.prob.set_val(Aircraft.Wing.SLAT_LIFT_INCREMENT_OPTIMUM, 0.930)
        self.prob.set_val("fus_lift", 0.05498)
        self.prob.set_val(Aircraft.Wing.FLAP_LIFT_INCREMENT_OPTIMUM, 1.500)
        self.prob.set_val(Aircraft.Wing.FLAP_DRAG_INCREMENT_OPTIMUM, 0.1)

        self.prob.set_val("flap_defl_ratio", 0.6666667)
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_UNWEIGHTED, 0.13966)
        self.prob.set_val("flap_defl", 40.0, units="deg")
        self.prob.set_val(Aircraft.Wing.FLAP_SPAN_RATIO, 0.65)

        self.prob.set_val("slat_defl_ratio", 0.5)
        self.prob.set_val("body_to_span_ratio", 0.09240447)
        self.prob.set_val("chord_to_body_ratio", 0.12679)
        self.prob.set_val(Aircraft.Wing.SLAT_SPAN_RATIO, 0.89759553)

    def test_case(self):

        self.prob.run_model()
        tol = 9e-4  # checked. high tol for lack of precision in GASP data.

        reg_data = 2.56197
        ans = self.prob["CL_max"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.18368
        ans = self.prob[Dynamic.Mission.MACH]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 164.78406
        ans = self.prob["reynolds"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.0362
        ans = self.prob["delta_CD"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.7816
        ans = self.prob["delta_CL"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(method="fd", out_stream=None)
        assert_check_partials(
            data, atol=6630, rtol=0.007
        )  # tolerance at this value for special cases. All other values check out.


class FlapsGroupTestCaseFowler(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.FLAP_TYPE, val=FlapType.FOWLER, units='unitless')

        self.prob.model = FCC = FlapsGroup(aviary_options=options)

        self.prob.setup()

        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Dynamic.Mission.TEMPERATURE, 518.67, units="degR")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.406626, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

        self.prob.set_val("VLAM1", 0.97217)
        self.prob.set_val("VLAM2", 1.09948)
        self.prob.set_val("VLAM3", 0.97217)
        self.prob.set_val("VLAM4", 1.25725)
        self.prob.set_val("VLAM5", 1.0000)
        self.prob.set_val("VLAM6", 1.1100)
        self.prob.set_val("VLAM7", 0.735)
        self.prob.set_val("VLAM8", 0.74444322)
        self.prob.set_val("VLAM9", 0.9975)
        self.prob.set_val("VLAM10", 0.74)
        self.prob.set_val("VLAM11", 0.84232)
        self.prob.set_val("VLAM12", 0.79208)
        self.prob.set_val("VLAM13", 1.03639)
        self.prob.set_val("VLAM14", 0.99142)

        self.prob.set_val("VDEL1", 1.0)
        self.prob.set_val("VDEL2", 0.64667)
        self.prob.set_val("VDEL3", 0.765)
        self.prob.set_val("VDEL4", 0.93578)
        self.prob.set_val("VDEL5", 0.90761)

        self.prob.set_val(Dynamic.Mission.SPEED_OF_SOUND, 1118.21948771, units="ft/s")
        self.prob.set_val(Aircraft.Wing.LOADING, 128.0, units="lbf/ft**2")
        self.prob.set_val(Dynamic.Mission.STATIC_PRESSURE,
                          (14.696 * 144), units="lbf/ft**2")
        self.prob.set_val(Aircraft.Wing.AVERAGE_CHORD, 12.61, units="ft")
        self.prob.set_val("kinematic_viscosity", 0.15723e-3, units="ft**2/s")
        self.prob.set_val(Aircraft.Wing.MAX_LIFT_REF, 1.150)
        self.prob.set_val(Aircraft.Wing.SLAT_LIFT_INCREMENT_OPTIMUM, 0.930)
        self.prob.set_val("fus_lift", 0.05498)
        self.prob.set_val(Aircraft.Wing.FLAP_LIFT_INCREMENT_OPTIMUM, 1.500)
        self.prob.set_val(Aircraft.Wing.FLAP_DRAG_INCREMENT_OPTIMUM, 0.1)

        self.prob.set_val("flap_defl_ratio", 1.333333)
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_UNWEIGHTED, 0.13966)
        self.prob.set_val("flap_defl", 40.0, units="deg")
        self.prob.set_val(Aircraft.Wing.FLAP_SPAN_RATIO, 0.65)

        self.prob.set_val("slat_defl_ratio", 0.5)
        self.prob.set_val("body_to_span_ratio", 0.09240447)
        self.prob.set_val("chord_to_body_ratio", 0.12679)
        self.prob.set_val(Aircraft.Wing.SLAT_SPAN_RATIO, 0.89759553)

    def test_case(self):

        self.prob.run_model()
        tol = 6e-4  # checked. high tol for lack of precision in GASP data.

        reg_data = 2.93271
        ans = self.prob["CL_max"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.17168
        ans = self.prob[Dynamic.Mission.MACH]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 154.02686
        ans = self.prob["reynolds"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.1070
        ans = self.prob["delta_CD"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 1.1441
        ans = self.prob["delta_CL"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(method="fd", out_stream=None)
        assert_check_partials(
            data, atol=6200, rtol=0.007
        )  # tolerance at this value for special cases. All other values check out.


class FlapsGroupTestCaseDoubleFowler(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.FLAP_TYPE,
                        val=FlapType.DOUBLE_SLOTTED_FOWLER, units='unitless')

        self.prob.model = FCC = FlapsGroup(aviary_options=options)

        self.prob.setup()

        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Dynamic.Mission.TEMPERATURE, 518.67, units="degR")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.406626, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

        self.prob.set_val("VLAM1", 0.97217)
        self.prob.set_val("VLAM2", 1.09948)
        self.prob.set_val("VLAM3", 0.97217)
        self.prob.set_val("VLAM4", 1.25725)
        self.prob.set_val("VLAM5", 1.0000)
        self.prob.set_val("VLAM6", 1.1100)
        self.prob.set_val("VLAM7", 0.735)
        self.prob.set_val("VLAM8", 0.74444322)
        self.prob.set_val("VLAM9", 0.9975)
        self.prob.set_val("VLAM10", 0.74)
        self.prob.set_val("VLAM11", 0.84232)
        self.prob.set_val("VLAM12", 0.79208)
        self.prob.set_val("VLAM13", 1.03639)
        self.prob.set_val("VLAM14", 0.99142)

        self.prob.set_val("VDEL1", 1.0)
        self.prob.set_val("VDEL2", 0.64667)
        self.prob.set_val("VDEL3", 0.765)
        self.prob.set_val("VDEL4", 0.93578)
        self.prob.set_val("VDEL5", 0.90761)

        self.prob.set_val(Dynamic.Mission.SPEED_OF_SOUND, 1118.21948771, units="ft/s")
        self.prob.set_val(Aircraft.Wing.LOADING, 128.0, units="lbf/ft**2")
        self.prob.set_val(Dynamic.Mission.STATIC_PRESSURE,
                          (14.696 * 144), units="lbf/ft**2")
        self.prob.set_val(Aircraft.Wing.AVERAGE_CHORD, 12.61, units="ft")
        self.prob.set_val("kinematic_viscosity", 0.15723e-3, units="ft**2/s")
        self.prob.set_val(Aircraft.Wing.MAX_LIFT_REF, 1.150)
        self.prob.set_val(Aircraft.Wing.SLAT_LIFT_INCREMENT_OPTIMUM, 0.930)
        self.prob.set_val("fus_lift", 0.05498)
        self.prob.set_val(Aircraft.Wing.FLAP_LIFT_INCREMENT_OPTIMUM, 1.500)
        self.prob.set_val(Aircraft.Wing.FLAP_DRAG_INCREMENT_OPTIMUM, 0.1)

        self.prob.set_val("flap_defl_ratio", 1.333333)
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_UNWEIGHTED, 0.13966)
        self.prob.set_val("flap_defl", 40.0, units="deg")
        self.prob.set_val(Aircraft.Wing.FLAP_SPAN_RATIO, 0.65)

        self.prob.set_val("slat_defl_ratio", 0.5)
        self.prob.set_val("body_to_span_ratio", 0.09240447)
        self.prob.set_val("chord_to_body_ratio", 0.12679)
        self.prob.set_val(Aircraft.Wing.SLAT_SPAN_RATIO, 0.89759553)

    def test_case(self):

        self.prob.run_model()
        tol = 6e-4  # checked. high tol for lack of precision in GASP data.

        reg_data = 2.93271
        ans = self.prob["CL_max"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.17168
        ans = self.prob[Dynamic.Mission.MACH]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 154.02686
        ans = self.prob["reynolds"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.1070
        ans = self.prob["delta_CD"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 1.1441
        ans = self.prob["delta_CL"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(method="fd", out_stream=None)
        assert_check_partials(
            data, atol=6200, rtol=0.007
        )  # tolerance at this value for special cases. All other values check out.


if __name__ == "__main__":
    unittest.main()



import numpy as np
from numpy import pi

from aviary.subsystems.propulsion.engine_deck import EngineDeck
from aviary.utils.aviary_values import AviaryValues
from aviary.utils.preprocessors import preprocess_propulsion
from aviary.utils.functions import get_path
from aviary.variable_info.enums import EquationsOfMotion, LegacyCode
from aviary.variable_info.variables import Aircraft, Mission, Settings

LargeSingleAisle2FLOPS = {}
inputs = LargeSingleAisle2FLOPS['inputs'] = AviaryValues()
outputs = LargeSingleAisle2FLOPS['outputs'] = AviaryValues()

'''
Large Single Aisle 2 Mission/fixed_gw
'''

# Overall Aircraft
# ---------------------------
inputs.set_val(Aircraft.Design.BASE_AREA, 0., 'ft**2')
inputs.set_val(Aircraft.Design.EMPTY_MASS_MARGIN_SCALER, 0.00514)
inputs.set_val(Aircraft.Design.LANDING_TO_TAKEOFF_MASS_RATIO, 0.84)
inputs.set_val(Mission.Design.GROSS_MASS, 174200., 'lbm')
inputs.set_val(Aircraft.Design.USE_ALT_MASS, False)
inputs.set_val(Aircraft.Design.LIFT_DEPENDENT_DRAG_COEFF_FACTOR, 1.0)
inputs.set_val(Aircraft.Design.SUBSONIC_DRAG_COEFF_FACTOR, 1.0)
inputs.set_val(Aircraft.Design.SUPERSONIC_DRAG_COEFF_FACTOR, 1.0)
inputs.set_val(Aircraft.Design.ZERO_LIFT_DRAG_COEFF_FACTOR, 0.945)

# Air Conditioning
# ---------------------------
inputs.set_val(Aircraft.AirConditioning.MASS_SCALER, 1.0)

# Anti-Icing
# ---------------------------
inputs.set_val(Aircraft.AntiIcing.MASS_SCALER, 1.0)

# APU
# ---------------------------
inputs.set_val(Aircraft.APU.MASS_SCALER, 1.0)

# Avionics
# ---------------------------
inputs.set_val(Aircraft.Avionics.MASS_SCALER, 1.0)

# Canard
# ---------------------------
inputs.set_val(Aircraft.Canard.AREA, 0.0, 'ft**2')
inputs.set_val(Aircraft.Canard.ASPECT_RATIO, 0.0)
inputs.set_val(Aircraft.Canard.THICKNESS_TO_CHORD, 0.0)

# Crew and Payload
# ---------------------------
inputs.set_val(Aircraft.CrewPayload.BAGGAGE_MASS_PER_PASSENGER, 35., 'lbm')
inputs.set_val(Aircraft.CrewPayload.NUM_BUSINESS_CLASS, 0)
inputs.set_val(Aircraft.CrewPayload.CARGO_CONTAINER_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.CrewPayload.NUM_FLIGHT_ATTENDANTS, 5)
inputs.set_val(Aircraft.CrewPayload.NUM_FLIGHT_CREW, 2)
inputs.set_val(Aircraft.CrewPayload.FLIGHT_CREW_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.CrewPayload.NUM_FIRST_CLASS, 12)
inputs.set_val(Aircraft.CrewPayload.NUM_GALLEY_CREW, 1)
inputs.set_val(Aircraft.CrewPayload.NUM_BUSINESS_CLASS, 0)
inputs.set_val(Aircraft.CrewPayload.MISC_CARGO, 4077., 'lbm')
inputs.set_val(Aircraft.CrewPayload.NON_FLIGHT_CREW_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.CrewPayload.NUM_TOURIST_CLASS, 150)
inputs.set_val(Aircraft.CrewPayload.NUM_PASSENGERS, 162, units='unitless')
inputs.set_val(Aircraft.CrewPayload.PASSENGER_SERVICE_MASS_SCALER, 1.)
inputs.set_val(Aircraft.CrewPayload.MASS_PER_PASSENGER, 165., 'lbm')
inputs.set_val(Aircraft.CrewPayload.WING_CARGO, 0., 'lbm')

# Electrical
# ---------------------------
inputs.set_val(Aircraft.Electrical.MASS_SCALER, 1.0)

# Fins
# ---------------------------
inputs.set_val(Aircraft.Fins.AREA, 0.0, 'ft**2')
inputs.set_val(Aircraft.Fins.NUM_FINS, 0)
inputs.set_val(Aircraft.Fins.TAPER_RATIO, 10.0)
inputs.set_val(Aircraft.Fins.MASS_SCALER, 1.0)

# Fuel
# ---------------------------
inputs.set_val(Aircraft.Fuel.AUXILIARY_FUEL_CAPACITY, 0.0, 'lbm')
inputs.set_val(Aircraft.Fuel.CAPACITY_FACTOR, 25.903)
inputs.set_val(Aircraft.Fuel.DENSITY_RATIO, 1.0)
inputs.set_val(Aircraft.Fuel.FUEL_SYSTEM_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Fuel.NUM_TANKS, 7)
inputs.set_val(Aircraft.Fuel.TOTAL_CAPACITY, 46063.0, 'lbm')
inputs.set_val(Aircraft.Fuel.UNUSABLE_FUEL_MASS_SCALER, 1.0)

# Furnishings
# ---------------------------
inputs.set_val(Aircraft.Furnishings.MASS_SCALER, 1.0)

# Fuselage
# ---------------------------
inputs.set_val(Aircraft.Fuselage.NUM_FUSELAGES, 1)
inputs.set_val(Aircraft.Fuselage.LENGTH, 124.75, 'ft')
inputs.set_val(Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH, 98.5, 'ft')
inputs.set_val(Aircraft.Fuselage.MAX_HEIGHT, 13.0208, 'ft')
inputs.set_val(Aircraft.Fuselage.MAX_WIDTH, 12.33, 'ft')
inputs.set_val(Aircraft.Fuselage.MILITARY_CARGO_FLOOR, False)
inputs.set_val(Aircraft.Fuselage.PLANFORM_AREA, 124.75 * 12.33, 'ft**2')
inputs.set_val(Aircraft.Fuselage.MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Fuselage.WETTED_AREA, 4142.317, 'ft**2')  # Override
inputs.set_val(Aircraft.Fuselage.WETTED_AREA_SCALER, 1.0)

# Horizontal Tail
# ---------------------------
inputs.set_val(Aircraft.HorizontalTail.AREA, 407.335370699457, 'ft**2')
inputs.set_val(Aircraft.HorizontalTail.ASPECT_RATIO, 5.444)
inputs.set_val(Aircraft.HorizontalTail.LAMINAR_FLOW_LOWER, 10.0)
inputs.set_val(Aircraft.HorizontalTail.LAMINAR_FLOW_UPPER, 10.0)
inputs.set_val(Aircraft.HorizontalTail.TAPER_RATIO, 0.3008)
inputs.set_val(Aircraft.HorizontalTail.THICKNESS_TO_CHORD, 0.1195)
inputs.set_val(Aircraft.HorizontalTail.VERTICAL_TAIL_FRACTION, 0.0)
inputs.set_val(Aircraft.HorizontalTail.MASS_SCALER, 1.0)
inputs.set_val(Aircraft.HorizontalTail.WETTED_AREA, 707.706, 'ft**2')  # Override
inputs.set_val(Aircraft.HorizontalTail.WETTED_AREA_SCALER, 1.0)

# Hydraulics
# ---------------------------
inputs.set_val(Aircraft.Hydraulics.SYSTEM_PRESSURE, 3000., 'lbf/ft**2')
inputs.set_val(Aircraft.Hydraulics.MASS_SCALER, 1.0)

# Instruments
# ---------------------------
inputs.set_val(Aircraft.Instruments.MASS_SCALER, 1.0)

# Landing Gear
# ---------------------------
inputs.set_val(Aircraft.LandingGear.CARRIER_BASED, False)
inputs.set_val(Aircraft.LandingGear.MAIN_GEAR_OLEO_LENGTH, 84.0, 'inch')
inputs.set_val(Aircraft.LandingGear.MAIN_GEAR_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.LandingGear.NOSE_GEAR_OLEO_LENGTH, 58.8, 'inch')
inputs.set_val(Aircraft.LandingGear.NOSE_GEAR_MASS_SCALER, 1.0)

# Nacelle
# ---------------------------
inputs.set_val(Aircraft.Nacelle.AVG_DIAMETER, 7.0, 'ft')
inputs.set_val(Aircraft.Nacelle.AVG_LENGTH, 11.65, 'ft')
inputs.set_val(Aircraft.Nacelle.MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Nacelle.WETTED_AREA_SCALER, 1.0)

# Paint
# ---------------------------
inputs.set_val(Aircraft.Paint.MASS_PER_UNIT_AREA, 0.07, 'lbm/ft**2')

# Propulsion and Engine
# ---------------------------
inputs.set_val(Aircraft.Propulsion.ENGINE_OIL_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Propulsion.MISC_MASS_SCALER, 1.0)

filename = get_path(
    'models/engines/turbofan_24k_1.deck')

engine_inputs = AviaryValues()
engine_inputs.set_val(Aircraft.Engine.DATA_FILE, filename)
engine_mass = 8071.35
engine_mass_units = 'lbm'
engine_inputs.set_val(Aircraft.Engine.MASS, engine_mass, engine_mass_units)
engine_inputs.set_val(
    Aircraft.Engine.REFERENCE_MASS,
    engine_mass,
    engine_mass_units)
scaled_sls_thrust = 27301.0
scaled_sls_thrust_units = 'lbf'
engine_inputs.set_val(Aircraft.Engine.SCALED_SLS_THRUST,
                      scaled_sls_thrust, scaled_sls_thrust_units)
engine_inputs.set_val(
    Aircraft.Engine.REFERENCE_SLS_THRUST, scaled_sls_thrust, scaled_sls_thrust_units)
num_engines = 2
engine_inputs.set_val(Aircraft.Engine.NUM_ENGINES, num_engines)
num_fuselage_engines = 0
engine_inputs.set_val(Aircraft.Engine.NUM_FUSELAGE_ENGINES, num_fuselage_engines)
num_wing_engines = 2
engine_inputs.set_val(Aircraft.Engine.NUM_WING_ENGINES, num_wing_engines)
engine_inputs.set_val(Aircraft.Engine.THRUST_REVERSERS_MASS_SCALER, 1.0)
engine_inputs.set_val(Aircraft.Engine.SCALE_MASS, True)
engine_inputs.set_val(Aircraft.Engine.MASS_SCALER, 1.15)
engine_inputs.set_val(Aircraft.Engine.SCALE_PERFORMANCE, True)
engine_inputs.set_val(Aircraft.Engine.SCALE_FACTOR, 1.0)
engine_inputs.set_val(Aircraft.Engine.SUBSONIC_FUEL_FLOW_SCALER, 1.0)
engine_inputs.set_val(Aircraft.Engine.SUPERSONIC_FUEL_FLOW_SCALER, 1.0)
engine_inputs.set_val(
    Aircraft.Engine.FUEL_FLOW_SCALER_CONSTANT_TERM, 0.0)
engine_inputs.set_val(Aircraft.Engine.FUEL_FLOW_SCALER_LINEAR_TERM, 0.0)
engine_inputs.set_val(Aircraft.Engine.CONSTANT_FUEL_CONSUMPTION, 0.0, units='lbm/h')
engine_inputs.set_val(Aircraft.Engine.ADDITIONAL_MASS_FRACTION, 0.0)
engine_inputs.set_val(Aircraft.Engine.GENERATE_FLIGHT_IDLE, True)
engine_inputs.set_val(Aircraft.Engine.IGNORE_NEGATIVE_THRUST, False)
engine_inputs.set_val(Aircraft.Engine.FLIGHT_IDLE_THRUST_FRACTION, 0.0)
engine_inputs.set_val(Aircraft.Engine.FLIGHT_IDLE_MAX_FRACTION, 1.0)
engine_inputs.set_val(Aircraft.Engine.FLIGHT_IDLE_MIN_FRACTION, 0.08)
engine_inputs.set_val(Aircraft.Engine.GEOPOTENTIAL_ALT, False)
engine_inputs.set_val(Aircraft.Engine.INTERPOLATION_METHOD, 'slinear')

# Vertical Tail
# ---------------------------
inputs.set_val(Aircraft.VerticalTail.NUM_TAILS, 1)
inputs.set_val(Aircraft.VerticalTail.AREA, 284.499779284585, 'ft**2')
inputs.set_val(Aircraft.VerticalTail.LAMINAR_FLOW_LOWER, 10.0)
inputs.set_val(Aircraft.VerticalTail.LAMINAR_FLOW_UPPER, 10.0)
inputs.set_val(Aircraft.VerticalTail.ASPECT_RATIO, 2.2262)
inputs.set_val(Aircraft.VerticalTail.TAPER_RATIO, 0.21082082638898)
inputs.set_val(Aircraft.VerticalTail.THICKNESS_TO_CHORD, 0.137459440381375)
inputs.set_val(Aircraft.VerticalTail.MASS_SCALER, 1.0)
inputs.set_val(Aircraft.VerticalTail.WETTED_AREA, 589.35, 'ft**2')  # Override
inputs.set_val(Aircraft.VerticalTail.WETTED_AREA_SCALER, 1.0)

# Wing
# ---------------------------
inputs.set_val(Aircraft.Wing.AEROELASTIC_TAILORING_FACTOR, 0.0)
inputs.set_val(Aircraft.Wing.AIRFOIL_TECHNOLOGY, 1.87)
inputs.set_val(Aircraft.Wing.AREA, 1341.0, 'ft**2')
inputs.set_val(Aircraft.Wing.ASPECT_RATIO, 9.45)
inputs.set_val(Aircraft.Wing.BENDING_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.COMPOSITE_FRACTION, 0.0)
inputs.set_val(Aircraft.Wing.CONTROL_SURFACE_AREA_RATIO, 0.333)
inputs.set_val(Aircraft.Wing.GLOVE_AND_BAT, 0.0, 'ft**2')
inputs.set_val(Aircraft.Wing.LAMINAR_FLOW_LOWER, 10.0)
inputs.set_val(Aircraft.Wing.LAMINAR_FLOW_UPPER, 10.0)
inputs.set_val(Aircraft.Wing.LOAD_FRACTION, 1.0)
inputs.set_val(Aircraft.Wing.MAX_CAMBER_AT_70_SEMISPAN, 0.015)
inputs.set_val(Aircraft.Wing.MISC_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.SHEAR_CONTROL_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.SPAN, 112.57, 'ft')
inputs.set_val(Aircraft.Wing.SPAN_EFFICIENCY_FACTOR, 1.35)
inputs.set_val(Aircraft.Wing.SPAN_EFFICIENCY_REDUCTION, False)
inputs.set_val(Aircraft.Wing.STRUT_BRACING_FACTOR, 0.0)
inputs.set_val(Aircraft.Wing.SURFACE_CONTROL_MASS_SCALER, 1.0)
inputs.set_val(Aircraft.Wing.SWEEP, 25.03, 'deg')
inputs.set_val(Aircraft.Wing.TAPER_RATIO, 0.237343146184852)
inputs.set_val(Aircraft.Wing.THICKNESS_TO_CHORD, 0.131732727515702)
inputs.set_val(Aircraft.Wing.ULTIMATE_LOAD_FACTOR, 3.75)
inputs.set_val(Aircraft.Wing.VAR_SWEEP_MASS_PENALTY, 0)
inputs.set_val(Aircraft.Wing.MASS, 15288, 'lbm')
inputs.set_val(Aircraft.Wing.WETTED_AREA, 2423.02, 'ft**2')  # Override
inputs.set_val(Aircraft.Wing.WETTED_AREA_SCALER, 1.0)

# Mission
# ---------------------------
inputs.set_val(Mission.Summary.CRUISE_MACH, 0.785)
inputs.set_val(Mission.Summary.FUEL_FLOW_SCALER, 1.0)
inputs.set_val(Mission.Design.RANGE, 2960., 'NM')
inputs.set_val(Mission.Constraints.MAX_MACH, 0.82)
# TODO investigate the origin of these values (taken from benchmark tests)
# TODO: where should this get connected from?
inputs.set_val(Mission.Takeoff.FUEL_SIMPLE, 659, 'lbm')
# TODO: should this come from aero?
inputs.set_val(Mission.Landing.LIFT_COEFFICIENT_MAX, 3)
# TODO: should this come from aero?
inputs.set_val(Mission.Takeoff.LIFT_COEFFICIENT_MAX, 2)
# TODO: should this come from aero?
inputs.set_val(Mission.Takeoff.LIFT_OVER_DRAG, 17.35)
# TODO: should this be a user input or should it be hard coded somewhere assuming it will
# never change?
inputs.set_val(Mission.Takeoff.ROLLING_FRICTION_COEFFICIENT, .0175)
# lbf TODO: where should this get connected from?
inputs.set_val(Mission.Design.THRUST_TAKEOFF_PER_ENG, 27301.0, 'lbf')

# Settings
# ---------------------------
inputs.set_val(Settings.EQUATIONS_OF_MOTION, EquationsOfMotion.HEIGHT_ENERGY)
inputs.set_val(Settings.MASS_METHOD, LegacyCode.FLOPS)

# ---------------------------
#          OUTPUTS
# ---------------------------

outputs.set_val(Aircraft.Design.EMPTY_MASS, 88246., 'lbm')
outputs.set_val(Aircraft.Design.EMPTY_MASS_MARGIN, 451.3, 'lbm')
outputs.set_val(Aircraft.Design.TOUCHDOWN_MASS, 146328.0, 'lbm')
outputs.set_val(Aircraft.Design.OPERATING_MASS, 95007., 'lbm')
outputs.set_val(Aircraft.Propulsion.MASS, 19232., 'lbm')
outputs.set_val(Aircraft.Design.STRUCTURE_MASS, 44389., 'lbm')
outputs.set_val(Aircraft.Design.SYSTEMS_EQUIP_MASS, 24174., 'lbm')
outputs.set_val(Aircraft.Design.TOTAL_WETTED_AREA, 8319.07, 'ft**2')
outputs.set_val(Aircraft.Design.ZERO_FUEL_MASS, 131484., 'lbm')
outputs.set_val(Mission.Design.FUEL_MASS, 42716., 'lbm')

outputs.set_val(Aircraft.AirConditioning.MASS, 1603.75, 'lbm')

outputs.set_val(Aircraft.AntiIcing.MASS, 195.93, 'lbm')

outputs.set_val(Aircraft.APU.MASS, 1014., 'lbm')

outputs.set_val(Aircraft.Avionics.MASS, 1339.4, 'lbm')

outputs.set_val(Aircraft.Canard.CHARACTERISTIC_LENGTH, 0.0, 'ft')
outputs.set_val(Aircraft.Canard.FINENESS, 0.0)
outputs.set_val(Aircraft.Canard.WETTED_AREA, 0.0, 'ft**2')
outputs.set_val(Aircraft.Canard.MASS, 0.0, 'lbm')

outputs.set_val(Aircraft.CrewPayload.BAGGAGE_MASS, 5670., 'lbm')
outputs.set_val(Aircraft.CrewPayload.CARGO_CONTAINER_MASS, 1925., 'lbm')
outputs.set_val(Aircraft.CrewPayload.CARGO_MASS, 4077., 'lbm')
outputs.set_val(Aircraft.CrewPayload.FLIGHT_CREW_MASS, 450.0, 'lbm')
outputs.set_val(Aircraft.CrewPayload.NON_FLIGHT_CREW_MASS, 975.0, 'lbm')
outputs.set_val(Aircraft.CrewPayload.PASSENGER_SERVICE_MASS, 2787.30285438, 'lbm')
outputs.set_val(Aircraft.CrewPayload.PASSENGER_MASS, 26730., 'lbm')
outputs.set_val(Aircraft.CrewPayload.TOTAL_PAYLOAD_MASS, 36477., 'lbm')

outputs.set_val(Aircraft.Electrical.MASS, 1935.6, 'lbm')

outputs.set_val(Aircraft.Fins.MASS, 0.0, 'lbm')

outputs.set_val(Aircraft.Fuel.FUEL_SYSTEM_MASS, 682.7, 'lbm')
outputs.set_val(Aircraft.Fuel.FUSELAGE_FUEL_CAPACITY, 3.9, 'lbm')
outputs.set_val(Aircraft.Fuel.UNUSABLE_FUEL_MASS, 497.7, 'lbm')

outputs.set_val(Aircraft.Furnishings.MASS, 14690., 'lbm')

avg_diameter = (13.0208 + 12.33) / 2
avg_diameter_units = 'ft'
outputs.set_val(Aircraft.Fuselage.AVG_DIAMETER, avg_diameter, avg_diameter_units)
outputs.set_val(Aircraft.Fuselage.CHARACTERISTIC_LENGTH, 124.75, 'ft')
outputs.set_val(
    Aircraft.Fuselage.CROSS_SECTION,
    pi * (avg_diameter / 2.0)**2.0, 'ft**2')
outputs.set_val(Aircraft.Fuselage.DIAMETER_TO_WING_SPAN, 0.112598)
outputs.set_val(Aircraft.Fuselage.FINENESS, 9.8419)
outputs.set_val(Aircraft.Fuselage.LENGTH_TO_DIAMETER, 9.841898)
outputs.set_val(Aircraft.Fuselage.MASS, 16790., 'lbm')

outputs.set_val(Aircraft.HorizontalTail.CHARACTERISTIC_LENGTH, 8.65, 'ft')
outputs.set_val(Aircraft.HorizontalTail.FINENESS, 0.1195)
outputs.set_val(Aircraft.HorizontalTail.MASS, 1931.8, 'lbm')

outputs.set_val(Aircraft.Hydraulics.MASS, 1075.3, 'lbm')

outputs.set_val(Aircraft.Hydraulics.SYSTEM_PRESSURE, 3000., 'lbf/ft**2')

outputs.set_val(Aircraft.Instruments.MASS, 484., 'lbm')

outputs.set_val(
    Aircraft.LandingGear.MAIN_GEAR_MASS, 0.0117 * 146328.0**0.95 * 84.0**0.43, 'lbm')
outputs.set_val(
    Aircraft.LandingGear.NOSE_GEAR_MASS, 0.048 * 146328.0**0.67 * 58.8**0.43, 'lbm')

outputs.set_val(Aircraft.Nacelle.CHARACTERISTIC_LENGTH, np.array([11.65]), 'ft')
outputs.set_val(Aircraft.Nacelle.FINENESS, np.array([1.6643]))
outputs.set_val(Aircraft.Nacelle.TOTAL_WETTED_AREA, 2 * 228.34, 'ft**2')
outputs.set_val(Aircraft.Nacelle.MASS, 1612.2, 'lbm')
outputs.set_val(Aircraft.Nacelle.WETTED_AREA, np.array([228.34]), 'ft**2')

outputs.set_val(Aircraft.Paint.MASS, 582.3, 'lbm')

outputs.set_val(
    Aircraft.Propulsion.TOTAL_SCALED_SLS_THRUST,
    scaled_sls_thrust * num_engines, scaled_sls_thrust_units)

total_num_engines = num_engines
outputs.set_val(Aircraft.Propulsion.TOTAL_NUM_ENGINES, total_num_engines)

ctrls_mass = 0.26 * 2 * 27301.0**0.5  # 85.92
ctrls_mass_units = 'lbm'

outputs.set_val(
    Aircraft.Engine.CONTROLS_MASS,
    ctrls_mass, ctrls_mass_units)
outputs.set_val(
    Aircraft.Propulsion.TOTAL_ENGINE_CONTROLS_MASS,
    ctrls_mass, ctrls_mass_units)

outputs.set_val(Aircraft.Propulsion.TOTAL_ENGINE_OIL_MASS, 125.42, 'lbm')

outputs.set_val(Aircraft.Propulsion.TOTAL_NUM_WING_ENGINES, num_wing_engines)

outputs.set_val(Aircraft.Propulsion.TOTAL_NUM_FUSELAGE_ENGINES, num_fuselage_engines)

outputs.set_val(
    Aircraft.Propulsion.TOTAL_ENGINE_MASS,
    engine_mass * num_engines, engine_mass_units)

outputs.set_val(Aircraft.Engine.MASS, 16143./2.0, 'lbm')
outputs.set_val(Aircraft.Engine.POD_MASS, 10350, 'lbm')
outputs.set_val(Aircraft.Engine.ADDITIONAL_MASS, 0.0, 'lbm')
outputs.set_val(Aircraft.Propulsion.TOTAL_MISC_MASS, 550.4, 'lbm')
outputs.set_val(Aircraft.Engine.SCALE_FACTOR, 1.0)
outputs.set_val(
    Aircraft.Propulsion.TOTAL_STARTER_MASS, 11.0 * 2 * 0.82**0.32 * 7.0**1.6, 'lbm')

thrust_reversers_mass = 1856.4
thrust_reversers_mass_units = 'lbm'
outputs.set_val(
    Aircraft.Propulsion.TOTAL_THRUST_REVERSERS_MASS,
    thrust_reversers_mass, thrust_reversers_mass_units)
outputs.set_val(
    Aircraft.Engine.THRUST_REVERSERS_MASS,
    thrust_reversers_mass, thrust_reversers_mass_units)

outputs.set_val(Aircraft.VerticalTail.CHARACTERISTIC_LENGTH, 11.30, 'ft')
outputs.set_val(Aircraft.VerticalTail.FINENESS, 0.1375)
outputs.set_val(Aircraft.VerticalTail.MASS, 1035.6, 'lbm')

outputs.set_val(Aircraft.Wing.BENDING_FACTOR, 8.8294)
outputs.set_val(Aircraft.Wing.BENDING_MASS, 6016.9, 'lbm')
outputs.set_val(Aircraft.Wing.CHARACTERISTIC_LENGTH, 11.91, 'ft')
outputs.set_val(Aircraft.Wing.CONTROL_SURFACE_AREA, 0.333 * 1341.0, 'ft**2')
outputs.set_val(Aircraft.Wing.ENG_POD_INERTIA_FACTOR, 0.940000)
outputs.set_val(Aircraft.Wing.FINENESS, 0.1317)
outputs.set_val(Aircraft.Wing.MISC_MASS, 1718.7, 'lbm')
outputs.set_val(Aircraft.Wing.SHEAR_CONTROL_MASS, 7552.6, 'lbm')
outputs.set_val(Aircraft.Wing.SURFACE_CONTROL_MASS, 1835., 'lbm')

outputs.set_val(Mission.Design.MACH, 0.799)
outputs.set_val(Mission.Design.LIFT_COEFFICIENT, 0.523)

# Create engine model
engine = EngineDeck(name='engine',
                    options=engine_inputs
                    )
preprocess_propulsion(inputs, [engine])



from aviary.models.large_single_aisle_2.large_single_aisle_2_altwt_FLOPS_data import \
    LargeSingleAisle2FLOPSalt
from aviary.models.large_single_aisle_2.large_single_aisle_2_detailwing_FLOPS_data import \
    LargeSingleAisle2FLOPSdw
from aviary.models.large_single_aisle_2.large_single_aisle_2_FLOPS_data import \
    LargeSingleAisle2FLOPS
from aviary.models.large_single_aisle_1.large_single_aisle_1_FLOPS_data import \
    LargeSingleAisle1FLOPS
from aviary.models.N3CC.N3CC_data import N3CC

FLOPS_Test_Data = {}

FLOPS_Test_Data['LargeSingleAisle1FLOPS'] = LargeSingleAisle1FLOPS
FLOPS_Test_Data['LargeSingleAisle2FLOPS'] = LargeSingleAisle2FLOPS
FLOPS_Test_Data['LargeSingleAisle2FLOPSdw'] = LargeSingleAisle2FLOPSdw
FLOPS_Test_Data['LargeSingleAisle2FLOPSalt'] = LargeSingleAisle2FLOPSalt
FLOPS_Test_Data['N3CC'] = N3CC



"""
This is a straightforward and basic example of running a coupled aircraft design-mission optimization in Aviary.
This uses the "level 1" API within Aviary.

We use the pre-defined single aisle commercial transport aircraft definition and use a pre-defined phase_info object
to describe the mission optimization problem to Aviary.
This mission consists of climb, cruise, and descent phases.
We then call the `run_aviary` function, which takes in the path to the aircraft model, the phase info, and some other options.
This performs a coupled design-mission optimization and outputs the results from Aviary into the `reports` folder.
"""
import aviary.api as av


prob = av.run_aviary('models/test_aircraft/aircraft_for_bench_FwFm.csv', av.default_height_energy_phase_info,
                     optimizer="SLSQP", make_plots=True)



import numpy as np
import openmdao.api as om

from aviary.constants import GRAV_ENGLISH_LBM
from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.functions import add_aviary_input, add_aviary_output
from aviary.variable_info.variables import Aircraft, Mission


class TransportFurnishingsGroupMass(om.ExplicitComponent):
    '''
    Calculates the mass of the furnishings group using the transport/general
    aviation method.
    '''

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.Furnishings.MASS_SCALER, val=1.0)

        add_aviary_input(self, Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH, val=0.0)

        add_aviary_input(self, Aircraft.Fuselage.MAX_WIDTH, val=0.0)

        add_aviary_input(self, Aircraft.Fuselage.MAX_HEIGHT, val=0.0)

        add_aviary_output(self, Aircraft.Furnishings.MASS, val=0.0)

    def setup_partials(self):
        self.declare_partials(of=Aircraft.Furnishings.MASS, wrt='*')

    def compute(
        self, inputs, outputs, discrete_inputs=None, discrete_outputs=None
    ):
        aviary_options: AviaryValues = self.options['aviary_options']

        flight_crew_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FLIGHT_CREW)
        first_class_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FIRST_CLASS)

        business_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_BUSINESS_CLASS)

        tourist_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_TOURIST_CLASS)

        fuse_count = aviary_options.get_val(Aircraft.Fuselage.NUM_FUSELAGES)

        scaler = inputs[Aircraft.Furnishings.MASS_SCALER]

        pax_compart_length = \
            inputs[Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH]

        fuse_max_width = inputs[Aircraft.Fuselage.MAX_WIDTH]
        fuse_max_height = inputs[Aircraft.Fuselage.MAX_HEIGHT]

        outputs[Aircraft.Furnishings.MASS] = (
            127.0 * flight_crew_count + 112.0 * first_class_count
            + 78.0 * business_class_count + 44.0 * tourist_class_count
            + 2.6 * pax_compart_length * (fuse_max_width + fuse_max_height)
            * fuse_count
        ) * scaler

    def compute_partials(self, inputs, J):
        aviary_options: AviaryValues = self.options['aviary_options']

        flight_crew_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FLIGHT_CREW)
        first_class_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FIRST_CLASS)

        business_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_BUSINESS_CLASS)

        tourist_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_TOURIST_CLASS)

        fuse_count = aviary_options.get_val(Aircraft.Fuselage.NUM_FUSELAGES)

        scaler = inputs[Aircraft.Furnishings.MASS_SCALER]
        fuse_max_width = inputs[Aircraft.Fuselage.MAX_WIDTH]
        fuse_max_height = inputs[Aircraft.Fuselage.MAX_HEIGHT]

        pax_compart_length = \
            inputs[Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH]

        J[Aircraft.Furnishings.MASS, Aircraft.Furnishings.MASS_SCALER] = (
            127.0 * flight_crew_count + 112.0 * first_class_count
            + 78.0 * business_class_count + 44.0 * tourist_class_count
            + 2.6 * pax_compart_length * (fuse_max_width + fuse_max_height)
            * fuse_count
        )

        J[
            Aircraft.Furnishings.MASS,
            Aircraft.Fuselage.PASSENGER_COMPARTMENT_LENGTH
        ] = 2.6 * scaler * (fuse_max_width + fuse_max_height) * fuse_count

        J[Aircraft.Furnishings.MASS, Aircraft.Fuselage.MAX_WIDTH] = \
            J[Aircraft.Furnishings.MASS, Aircraft.Fuselage.MAX_HEIGHT] = \
            2.6 * scaler * pax_compart_length * fuse_count


class BWBFurnishingsGroupMass(om.ExplicitComponent):
    '''
    Calculates the mass of the furnishings group for HWB aircraft.
    '''

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.Furnishings.MASS_SCALER, val=1.0)

        add_aviary_input(self, Aircraft.BWB.CABIN_AREA, val=100.0)

        add_aviary_input(self, Aircraft.Fuselage.MAX_WIDTH, val=30.0)

        add_aviary_input(self, Aircraft.Fuselage.MAX_HEIGHT, val=15.0)

        add_aviary_input(
            self, Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP, val=45.0
        )

        add_aviary_output(self, Aircraft.Furnishings.MASS, val=0.0)

    def setup_partials(self):
        self.declare_partials(of=Aircraft.Furnishings.MASS, wrt='*')

    def compute(
        self, inputs, outputs, discrete_inputs=None, discrete_outputs=None
    ):
        aviary_options: AviaryValues = self.options['aviary_options']

        flight_crew_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FLIGHT_CREW)
        first_class_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FIRST_CLASS)

        business_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_BUSINESS_CLASS)

        tourist_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_TOURIST_CLASS)

        scaler = inputs[Aircraft.Furnishings.MASS_SCALER]
        fuse_max_width = inputs[Aircraft.Fuselage.MAX_WIDTH]
        fuse_max_height = inputs[Aircraft.Fuselage.MAX_HEIGHT]

        weight = (
            127.0 * flight_crew_count + 112.0 * first_class_count
            + 78.0 * business_class_count + 44.0 * tourist_class_count
        )
        # outputs[Aircraft.Furnishings.MASS] = weight / GRAV_ENGLISH_LBM

        if not aviary_options.get_val(Aircraft.Fuselage.MILITARY_CARGO_FLOOR):
            acabin = inputs[Aircraft.BWB.CABIN_AREA]
            nbay = aviary_options.get_val(Aircraft.BWB.NUM_BAYS)

            cos = np.cos(
                np.pi/180*(inputs[Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP])
            )

            weight += 2.6 * (
                (acabin / fuse_max_width)
                * (fuse_max_width + fuse_max_height * nbay)
                + (fuse_max_width * (1.0 + 1.0 / cos) * fuse_max_height)
            )

        outputs[Aircraft.Furnishings.MASS] = weight * scaler / GRAV_ENGLISH_LBM

    def compute_partials(self, inputs, J):
        aviary_options: AviaryValues = self.options['aviary_options']

        flight_crew_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FLIGHT_CREW)
        first_class_count = aviary_options.get_val(Aircraft.CrewPayload.NUM_FIRST_CLASS)

        business_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_BUSINESS_CLASS)

        tourist_class_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_TOURIST_CLASS)

        scaler = inputs[Aircraft.Furnishings.MASS_SCALER]

        J[Aircraft.Furnishings.MASS, Aircraft.Furnishings.MASS_SCALER] = (
            127.0 * flight_crew_count + 112.0 * first_class_count
            + 78.0 * business_class_count + 44.0 * tourist_class_count
        ) / GRAV_ENGLISH_LBM

        if aviary_options.get_val(Aircraft.Fuselage.MILITARY_CARGO_FLOOR):
            J[Aircraft.Furnishings.MASS, Aircraft.BWB.CABIN_AREA] = 0.0

            J[Aircraft.Furnishings.MASS, Aircraft.Fuselage.MAX_WIDTH] = 0.0

            J[Aircraft.Furnishings.MASS, Aircraft.Fuselage.MAX_HEIGHT] = 0.0

            J[
                Aircraft.Furnishings.MASS,
                Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP
            ] = 0.0

        else:
            d2r = np.radians(inputs[Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP])
            cos = np.cos(d2r)
            tan = np.tan(d2r)

            acabin = inputs[Aircraft.BWB.CABIN_AREA]
            nbay = aviary_options.get_val(Aircraft.BWB.NUM_BAYS)
            fuse_max_width = inputs[Aircraft.Fuselage.MAX_WIDTH]
            fuse_max_height = inputs[Aircraft.Fuselage.MAX_HEIGHT]
            cabin_area = inputs[Aircraft.BWB.CABIN_AREA]

            J[
                Aircraft.Furnishings.MASS,
                Aircraft.Furnishings.MASS_SCALER
            ] += 2.6 * (
                (acabin / fuse_max_width)
                * (fuse_max_width + fuse_max_height * nbay)
                + (fuse_max_width * (1.0 + 1.0 / cos) * fuse_max_height)
            ) / GRAV_ENGLISH_LBM

            J[Aircraft.Furnishings.MASS, Aircraft.BWB.CABIN_AREA] = (
                2.6 * scaler
                * (fuse_max_width + fuse_max_height * nbay)
                / fuse_max_width
            ) / GRAV_ENGLISH_LBM

            J[Aircraft.Furnishings.MASS, Aircraft.Fuselage.MAX_WIDTH] = (
                2.6 * scaler * (
                    -cabin_area * fuse_max_height * nbay
                    / (fuse_max_width * fuse_max_width)
                    + (1.0 + 1.0 / cos) * fuse_max_height
                )
            ) / GRAV_ENGLISH_LBM

            J[Aircraft.Furnishings.MASS, Aircraft.Fuselage.MAX_HEIGHT] = (
                2.6 * scaler * (
                    (cabin_area / fuse_max_width) * nbay
                    + fuse_max_width * (1.0 + 1.0 / cos)
                )
            ) / GRAV_ENGLISH_LBM

            J[
                Aircraft.Furnishings.MASS,
                Aircraft.BWB.PASSENGER_LEADING_EDGE_SWEEP
            ] = (
                2.6 * scaler * fuse_max_width * fuse_max_height
                * tan / cos / (180 / np.pi)
            ) / GRAV_ENGLISH_LBM


class AltFurnishingsGroupMassBase(om.ExplicitComponent):
    '''
    Calculates the base mass of the furnishings group using the alternate
    method. The methodology is based on the FLOPS weight equations,
    modified to output mass instead of weight.
    '''

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.Furnishings.MASS_SCALER, val=1.0)

        add_aviary_output(self, Aircraft.Furnishings.MASS_BASE, val=0.0)

    def setup_partials(self):
        self.declare_partials(of=Aircraft.Furnishings.MASS_BASE, wrt='*')

    def compute(
        self, inputs, outputs, discrete_inputs=None, discrete_outputs=None
    ):
        aviary_options: AviaryValues = self.options['aviary_options']
        pax_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_PASSENGERS, units='unitless')
        scaler = inputs[Aircraft.Furnishings.MASS_SCALER]

        outputs[Aircraft.Furnishings.MASS_BASE] = \
            (82.15 * pax_count + 3600.0) * scaler

    def compute_partials(self, inputs, J, discrete_inputs=None):
        aviary_options: AviaryValues = self.options['aviary_options']
        pax_count = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_PASSENGERS, units='unitless')

        J[
            Aircraft.Furnishings.MASS_BASE,
            Aircraft.Furnishings.MASS_SCALER
        ] = 82.15 * pax_count + 3600.0


class AltFurnishingsGroupMass(om.ExplicitComponent):
    '''
    Completes the mass calculation for the furnishings group using the
    alternate method. The methodology is based on the FLOPS weight
    equations, modified to output mass instead of weight.
    '''

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.Furnishings.MASS_BASE, val=0.0)

        add_aviary_input(self, Aircraft.Design.STRUCTURE_MASS, val=0.0)

        add_aviary_input(self, Aircraft.Propulsion.MASS, val=0.0)

        add_aviary_input(self, Aircraft.Design.SYSTEMS_EQUIP_MASS_BASE, val=0.0)

        add_aviary_output(self, Aircraft.Furnishings.MASS, val=0.0)

    def setup_partials(self):
        self.declare_partials(
            of=Aircraft.Furnishings.MASS,
            wrt=[
                Aircraft.Design.STRUCTURE_MASS,
                Aircraft.Propulsion.MASS,
                Aircraft.Design.SYSTEMS_EQUIP_MASS_BASE
            ],
            val=0.01
        )

        self.declare_partials(
            of=Aircraft.Furnishings.MASS,
            wrt=Aircraft.Furnishings.MASS_BASE,
            val=1.
        )

    def compute(
        self, inputs, outputs, discrete_inputs=None, discrete_outputs=None
    ):
        furn_mass_base = inputs[Aircraft.Furnishings.MASS_BASE]
        struct_mass = inputs[Aircraft.Design.STRUCTURE_MASS]
        prop_mass = inputs[Aircraft.Propulsion.MASS]
        syseq_mass_base = inputs[Aircraft.Design.SYSTEMS_EQUIP_MASS_BASE]

        outputs[Aircraft.Furnishings.MASS] = furn_mass_base \
            + 0.01 * (struct_mass + prop_mass + syseq_mass_base)



import argparse
import glob
import json
import os
from pathlib import Path
import pathlib
import shutil
import importlib.util

import numpy as np
from bokeh.palettes import Category10
import hvplot.pandas  # noqa # need this ! Otherwise hvplot using DataFrames does not work
import pandas as pd
import panel as pn
from panel.theme import DefaultTheme

import openmdao.api as om
from openmdao.utils.general_utils import env_truthy

pn.extension(sizing_mode="stretch_width")

# Constants - # Can't get using CSS to work with frames and the raw_css for the template so going with
#    this for now
aviary_variables_json_file_name = 'aviary_vars.json'


def _dashboard_setup_parser(parser):
    """
    Set up the aviary subparser for the 'aviary dashboard' command.

    Parameters
    ----------
    parser : argparse subparser
        The parser we're adding options to.
    """
    parser.add_argument(
        "script_name",
        type=str,
        help="Name of aviary script that was run (not including .py).",
    )

    parser.add_argument(
        "--problem_recorder",
        type=str,
        help="Problem case recorder file name",
        dest="problem_recorder",
        default="aviary_history.db",
    )
    parser.add_argument(
        "--driver_recorder",
        type=str,
        help="Driver case recorder file name",
        dest="driver_recorder",
        default=None,
    )
    parser.add_argument(
        "--port",
        dest="port",
        type=int,
        default=5000,
        help="dashboard server port ID (default is 5000)",
    )

    # For future use
    parser.add_argument(
        "-d",
        "--debug",
        action="store_true",
        dest="debug_output",
        help="show debugging output",
    )


def _dashboard_cmd(options, user_args):
    """
    Run the dashboard command.

    Parameters
    ----------
    options : argparse Namespace
        Command line options.
    user_args : list of str
        Args to be passed to the user script.
    """
    dashboard(
        options.script_name,
        options.problem_recorder,
        options.driver_recorder,
        options.port,
    )


def create_report_frame(format, text_filepath):
    """
    Create a Panel Pane that contains an embedded external file in HTML, Markdown, or text format.

    Parameters
    ----------
    format : str
        Format of the file to be embeded. Options are 'html', 'text', 'markdown'.
    text_file_name : str
        Name of the report text file.

    Returns
    -------
    pane : Panel.Pane or None
        A Panel Pane object to be displayed in the dashboard. Or None if the file
        does not exist.
    """
    if os.path.exists(text_filepath):
        if format == 'html':
            iframe_css = 'width=1200px height=800px overflow-x="scroll" overflow="scroll" margin=0px padding=0px border=20px frameBorder=20px scrolling="yes"'
            report_pane = pn.pane.HTML(
                f'<iframe {iframe_css} src=/home/{text_filepath}></iframe>')
        elif format in ['markdown', 'text']:
            with open(text_filepath, "rb") as f:
                file_text = f.read()
                # need to deal with some encoding errors
                file_text = file_text.decode("latin-1")
            if format == "markdown":
                report_pane = pn.pane.Markdown(file_text)
            elif format == "text":
                report_pane = pn.pane.Markdown(f"```\n{file_text}\n```\n")
        else:
            raise RuntimeError(f"Report format of {format} is not supported.")
    else:
        report_pane = None
    return report_pane


def create_aviary_variables_table_data_nested(script_name, recorder_file):
    """
    Create a JSON file with information about Aviary variables.

    The JSON file has one level of hierarchy of the variables. The file
    is written to aviary_vars.json. That file is then read in by the
    aviary/visualization/assets/aviary_vars/script.js script. That is inside the
    aviary/visualization/assets/aviary_vars/index.html file that is embedded in the
    dashboard.

    The information about the variables comes from a case recorder file.

    Parameters
    ----------
    recorder_file : str
        Name of the recorder file containing the Problem cases.

    Returns
    -------
    table_data_nested
        A nested list of information about the Aviary variables.

    """
    cr = om.CaseReader(recorder_file)
    if 'final' not in cr.list_cases():
        return None

    case = cr.get_case('final')
    outputs = case.list_outputs(explicit=True, implicit=True, val=True,
                                residuals=True, residuals_tol=None,
                                units=True, shape=True, bounds=True, desc=True,
                                scaling=False, hierarchical=True, print_arrays=True,
                                out_stream=None, return_format='dict')

    sorted_abs_names = sorted(outputs.keys())

    grouped = {}
    for s in sorted_abs_names:
        prefix = s.split(":")[0]
        if prefix not in grouped:
            grouped[prefix] = []
        grouped[prefix].append(s)

    sorted_group_names = sorted(grouped.keys())

    table_data_nested = []
    for group_name in sorted_group_names:
        if len(grouped[group_name]) == 1:  # a list of one var.
            var_info = grouped[group_name][0]
            table_data_nested.append(
                {
                    "abs_name": group_name,
                    "prom_name": outputs[var_info]["prom_name"],
                    "value": str(outputs[var_info]["val"]),
                    "units": str(outputs[var_info]["units"]),
                }
            )
        else:
            # create children
            children_list = []
            for children_name in grouped[group_name]:
                var_info = outputs[children_name]
                children_list.append(
                    {
                        "abs_name": children_name,
                        "prom_name": outputs[children_name]["prom_name"],
                        "value": str(outputs[children_name]["val"]),
                        "units": str(outputs[children_name]["units"]),
                    }
                )
            table_data_nested.append(  # not a real var, just a group of vars so no values
                {
                    "abs_name": group_name,
                    "prom_name": "",
                    "value": "",
                    "units": "",
                    "_children": children_list,
                }
            )

    aviary_variables_file_path = (
        f"reports/{script_name}/aviary_vars/{aviary_variables_json_file_name}"
    )
    with open(aviary_variables_file_path, "w") as fp:
        json.dump(table_data_nested, fp)

    return table_data_nested


def convert_case_recorder_file_to_df(recorder_file_name):
    """
    Convert a case recorder file into a Pandas data frame.

    Parameters
    ----------
    recorder_file_name : str
        Name of the case recorder file.
    """
    cr = om.CaseReader(recorder_file_name)
    driver_cases = cr.list_cases("driver", out_stream=None)

    df = None
    for i, case in enumerate(driver_cases):
        driver_case = cr.get_case(case)

        desvars = driver_case.get_design_vars(scaled=False)
        objectives = driver_case.get_objectives(scaled=False)
        constraints = driver_case.get_constraints(scaled=False)

        if i == 0:  # Only need to get header of the data frame once
            # Need to worry about the fact that a variable can be in more than one of
            #  desvars, cons, and obj. So filter out the dupes
            initial_desvars_names = list(desvars.keys())
            initial_constraints_names = list(constraints.keys())
            objectives_names = list(objectives.keys())

            # Start with obj, then cons, then desvars
            # Give priority to having a duplicate being in the obj and cons
            #  over being in the desvars
            all_var_names = objectives_names.copy()
            constraints_names = []
            for name in initial_constraints_names:
                if name not in all_var_names:
                    constraints_names.append(name)
                    all_var_names.append(name)
            desvars_names = []
            for name in initial_desvars_names:
                if name not in all_var_names:
                    desvars_names.append(name)
                    all_var_names.append(name)
            header = ["iter_count"] + all_var_names
            df = pd.DataFrame(columns=header)

        # Now fill up a row
        row = [
            i,
        ]
        # important to do in this order since that is the order added to the header
        for varname in objectives_names:
            value = objectives[varname]
            if not np.isscalar(value):
                value = np.linalg.norm(value)
            row.append(value)

        for varname in constraints_names:
            value = constraints[varname]
            if not np.isscalar(value):
                value = np.linalg.norm(value)
            row.append(value)

        for varname in desvars_names:
            value = desvars[varname]
            if not np.isscalar(value):
                value = np.linalg.norm(value)
            row.append(value)
        df.loc[i] = row

    return df


def dashboard(script_name, problem_recorder, driver_recorder, port):
    """
    Generate the dashboard app display.

    Parameters
    ----------
    script_name : str
        Name of the script file whose results will be displayed by this dashboard.
    problem_recorder : str
        Name of the recorder file containing the Problem cases.
    driver_recorder : str
        Name of the recorder file containing the Driver cases.
    """
    reports_dir = f"reports/{script_name}/"

    if not Path(reports_dir).is_dir():
        raise ValueError(
            f"The script name, '{script_name}', does not have a reports folder associated with it. "
            f"The directory '{reports_dir}' does not exist."
        )

    # TODO - use lists and functions to do this with a lot less code
    ####### Model Tab #######
    model_tabs_list = []

    # Inputs
    inputs_pane = create_report_frame("html", f"{reports_dir}/inputs.html")
    if inputs_pane:
        model_tabs_list.append(('Inputs', inputs_pane))

    #  Debug Input List
    input_list_pane = create_report_frame("text", "input_list.txt")
    if input_list_pane:
        model_tabs_list.append(('Debug Input List', input_list_pane))

    #  Debug Output List
    output_list_pane = create_report_frame("text", "output_list.txt")
    if output_list_pane:
        model_tabs_list.append(('Debug Output List', output_list_pane))

    # N2
    n2_pane = create_report_frame("html", f"{reports_dir}/n2.html")
    if n2_pane:
        model_tabs_list.append(('N2', n2_pane))

    # Trajectory Linkage
    traj_linkage_report_pane = create_report_frame(
        "html", f"{reports_dir}/traj_linkage_report.html"
    )
    if traj_linkage_report_pane:
        model_tabs_list.append(('Trajectory Linkage Report',
                                traj_linkage_report_pane))

    ####### Optimization Tab #######
    optimization_tabs_list = []

    # Driver scaling
    driver_scaling_report_pane = create_report_frame(
        "html", f"{reports_dir}/driver_scaling_report.html"
    )
    if driver_scaling_report_pane:
        optimization_tabs_list.append(
            ("Driver Scaling Report", driver_scaling_report_pane)
        )

    # Coloring report
    coloring_report_pane = create_report_frame(
        "html", f"{reports_dir}/total_coloring.html"
    )
    if coloring_report_pane:
        optimization_tabs_list.append(("Total Coloring Report", coloring_report_pane))

    # Optimization report
    opt_report_pane = create_report_frame("html", f"{reports_dir}/opt_report.html")
    if opt_report_pane:
        optimization_tabs_list.append(("Optimization Report", opt_report_pane))

    # IPOPT report
    ipopt_pane = create_report_frame("text", f"{reports_dir}/IPOPT.out")
    if ipopt_pane:
        optimization_tabs_list.append(("IPOPT Output", ipopt_pane))

    # SNOPT report
    snopt_pane = create_report_frame("text", f"{reports_dir}/SNOPT_print.out")
    if snopt_pane:
        optimization_tabs_list.append(("SNOPT Output", snopt_pane))

    # SNOPT summary
    snopt_summary_pane = create_report_frame("text", f"{reports_dir}/SNOPT_summary.out")
    if snopt_summary_pane:
        optimization_tabs_list.append(("SNOPT Summary", snopt_summary_pane))

    # PyOpt report
    pyopt_solution_pane = create_report_frame(
        "text", f"{reports_dir}/pyopt_solution.txt"
    )
    if pyopt_solution_pane:
        optimization_tabs_list.append(("PyOpt Solution", pyopt_solution_pane))

    # Desvars, cons, opt interactive plot
    if driver_recorder:
        if os.path.exists(driver_recorder):
            df = convert_case_recorder_file_to_df(f"{driver_recorder}")
            if df is not None:
                variables = pn.widgets.CheckBoxGroup(
                    name="Variables",
                    options=list(df.columns),
                    # just so all of them aren't plotted from the beginning. Skip the iter count
                    value=list(df.columns)[1:2],
                )
                ipipeline = df.interactive()
                ihvplot = ipipeline.hvplot(
                    y=variables,
                    responsive=True,
                    min_height=400,
                    color=list(Category10[10]),
                    yformatter="%.0f",
                    title="Model Optimization using OpenMDAO",
                )
                optimization_plot_pane = pn.Column(
                    pn.Row(
                        pn.Column(
                            variables,
                            pn.VSpacer(height=30),
                            pn.VSpacer(height=30),
                            width=300,
                        ),
                        ihvplot.panel(),
                    )
                )
                optimization_tabs_list.append(
                    ("Desvars, cons, opt", optimization_plot_pane)
                )
            else:
                optimization_tabs_list.append(
                    (
                        "Desvars, cons, opt",
                        pn.pane.Markdown(
                            f"# Recorder file '{driver_recorder}' does not have Driver case recordings"
                        ),
                    )
                )
        else:
            optimization_tabs_list.append(
                (
                    "Desvars, cons, opt",
                    pn.pane.Markdown(f"# Recorder file '{driver_recorder}' not found"),
                )
            )

    ####### Results Tab #######
    results_tabs_list = []

    # Trajectory results
    traj_results_report_pane = create_report_frame(
        "html", f"{reports_dir}/traj_results_report.html"
    )
    if traj_results_report_pane:
        results_tabs_list.append(
            ("Trajectory Results Report", traj_results_report_pane)
        )

    # Make the Aviary variables table pane
    if problem_recorder:
        if os.path.exists(problem_recorder):
            # Make dir reports/script_name/aviary_vars if needed
            aviary_vars_dir = pathlib.Path(f"reports/{script_name}/aviary_vars")
            aviary_vars_dir.mkdir(parents=True, exist_ok=True)

            # copy index.html file to reports/script_name/aviary_vars/index.html
            aviary_dir = pathlib.Path(importlib.util.find_spec("aviary").origin).parent

            shutil.copy(
                aviary_dir.joinpath("visualization/assets/aviary_vars/index.html"),
                aviary_vars_dir.joinpath("index.html"),
            )
            shutil.copy(
                aviary_dir.joinpath("visualization/assets/aviary_vars/script.js"),
                aviary_vars_dir.joinpath("script.js"),
            )
            # copy script.js file to reports/script_name/aviary_vars/index.html.
            # mod the script.js file to point at the json file
            # create the json file and put it in reports/script_name/aviary_vars/aviary_vars.json
            create_aviary_variables_table_data_nested(
                script_name, problem_recorder
            )  # create the json file
            aviary_vars_pane = create_report_frame(
                "html", f"{reports_dir}/aviary_vars/index.html"
            )

            results_tabs_list.append(("Aviary Variables", aviary_vars_pane))

    ####### Subsystems Tab #######
    subsystem_tabs_list = []

    # Look through subsystems directory for markdown files
    for md_file in Path(f"{reports_dir}subsystems").glob("*.md"):
        example_subsystems_pane = create_report_frame("markdown", str(md_file))
        subsystem_tabs_list.append((md_file.stem, example_subsystems_pane))

    model_tabs = pn.Tabs(*model_tabs_list, stylesheets=['assets/aviary_styles.css'])
    optimization_tabs = pn.Tabs(*optimization_tabs_list,
                                stylesheets=['assets/aviary_styles.css'])
    results_tabs = pn.Tabs(*results_tabs_list, stylesheets=['assets/aviary_styles.css'])
    if subsystem_tabs_list:
        subsystem_tabs = pn.Tabs(*subsystem_tabs_list,
                                 stylesheets=['assets/aviary_styles.css'])

    # Add subtabs to tabs
    high_level_tabs = []
    high_level_tabs.append(('Model', model_tabs))
    high_level_tabs.append(('Optimization', optimization_tabs))
    high_level_tabs.append(('Results', results_tabs))
    if subsystem_tabs_list:
        high_level_tabs.append(('Subsystems', subsystem_tabs))
    tabs = pn.Tabs(*high_level_tabs, stylesheets=['assets/aviary_styles.css'])

    template = pn.template.FastListTemplate(
        title=f"Aviary Dashboard for {script_name}",
        logo="assets/aviary_logo.png",
        favicon="assets/aviary_logo.png",
        main=[tabs],
        accent_base_color="black",
        header_background="rgb(0, 212, 169)",
        background_color="white",
        theme=DefaultTheme,
        theme_toggle=False,
        main_layout=None,
        css_files=['assets/aviary_styles.css']
    )

    if env_truthy("TESTFLO_RUNNING"):
        show = False
        threaded = True
    else:
        show = True
        threaded = False

    assets_dir = pathlib.Path(
        importlib.util.find_spec("aviary").origin
    ).parent.joinpath("visualization/assets/")
    home_dir = "."
    server = pn.serve(
        template,
        port=port,
        address="localhost",
        websocket_origin=f"localhost:{port}",
        show=show,
        threaded=threaded,
        static_dirs={
            "reports": reports_dir,
            "home": home_dir,
            "assets": assets_dir,
        },
    )
    server.stop()


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    _dashboard_setup_parser(parser)
    args = parser.parse_args()
    _dashboard_cmd(args, None)



import numpy as np
import openmdao.api as om

import aviary.constants as constants
from aviary.variable_info.variables import Dynamic


class AltitudeRate(om.ExplicitComponent):
    """
    Rutowski "Energy Approach to the General Aircraft Performance Problem", doi 10.2514/8.2956
    Equation 6
    """

    def initialize(self):
        self.options.declare('num_nodes', types=int)

    def setup(self):
        nn = self.options['num_nodes']
        self.add_input(Dynamic.Mission.SPECIFIC_ENERGY_RATE, val=np.ones(
            nn), desc='current specific power', units='m/s')
        self.add_input(Dynamic.Mission.VELOCITY_RATE, val=np.ones(
            nn), desc='current acceleration', units='m/s**2')
        self.add_input(
            Dynamic.Mission.VELOCITY,
            val=np.ones(nn),
            desc='current velocity',
            units='m/s')
        self.add_output(Dynamic.Mission.ALTITUDE_RATE, val=np.ones(
            nn), desc='current climb rate', units='m/s')

    def compute(self, inputs, outputs):
        gravity = constants.GRAV_METRIC_FLOPS
        specific_power = inputs[Dynamic.Mission.SPECIFIC_ENERGY_RATE]
        acceleration = inputs[Dynamic.Mission.VELOCITY_RATE]
        velocity = inputs[Dynamic.Mission.VELOCITY]

        outputs[Dynamic.Mission.ALTITUDE_RATE] = \
            specific_power - (velocity * acceleration) / gravity

    def setup_partials(self):
        arange = np.arange(self.options['num_nodes'])
        self.declare_partials(Dynamic.Mission.ALTITUDE_RATE,
                              [Dynamic.Mission.SPECIFIC_ENERGY_RATE,
                               Dynamic.Mission.VELOCITY_RATE,
                               Dynamic.Mission.VELOCITY],
                              rows=arange,
                              cols=arange,
                              val=1)

    def compute_partials(self, inputs, J):
        gravity = constants.GRAV_METRIC_FLOPS
        acceleration = inputs[Dynamic.Mission.VELOCITY_RATE]
        velocity = inputs[Dynamic.Mission.VELOCITY]

        J[Dynamic.Mission.ALTITUDE_RATE, Dynamic.Mission.VELOCITY_RATE] = -velocity / gravity
        J[Dynamic.Mission.ALTITUDE_RATE, Dynamic.Mission.VELOCITY] = -acceleration / gravity



from setuptools import find_packages, setup
from pathlib import Path
import re


# Version info is set in one place; the aviary/__init__.py file
__version__ = re.findall(
    r"""__version__ = ["']+([0-9\.\-dev]*)["']+""",
    open('aviary/__init__.py').read(),
)[0]

with open(Path(__file__).parent / "README.md", encoding="utf-8") as f:
    long_description = f.read()

pkgname = "aviary"
extras_require = {
    "test": ["testflo", "pre-commit"],
    "examples": ["openaerostruct", "ambiance", "itables"],
}

all_packages = []
for packages in extras_require.values():
    all_packages.extend(packages)

extras_require["all"] = all_packages

setup(
    name="om-aviary",
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=__version__,
    packages=find_packages(),
    install_requires=[
        "openmdao>=3.27.0",
        "dymos>=1.8.1",
        "hvplot",
        "numpy",
        "matplotlib",
        "pandas",
        "panel>=1.0.0",
        "parameterized",
        "simupy",
    ],
    extras_require=extras_require,
    package_data={
        pkgname: [
            "subsystems/aerodynamics/gasp_based/data/*",
            "subsystems/aerodynamics/gasp_based/test/data/*",
            "subsystems/aerodynamics/flops_based/test/*.csv",
            "subsystems/aerodynamics/flops_based/test/data/*.csv",
            "subsystems/propulsion/gasp_based/data/*",
            "subsystems/propulsion/test/*.csv",
            "validation_cases/validation_data/gasp_data/*.dat",
            "validation_cases/validation_data/gasp_data/*.csv",
            "validation_cases/validation_data/flops_data/engine_only/*.deck",
            "utils/legacy_code_data/*default_values*",
            "utils/test/*",
            "models/engines/*.deck",
            "models/engines/*.txt",
            "models/N3CC/*",
            "models/large_single_aisle_1/*",
            "models/large_single_aisle_2/*",
            "models/small_single_aisle/*",
            "models/test_aircraft/*",
            "visualization/assets/*",
            "visualization/assets/aviary_vars/*"
        ],
        f"{pkgname}.docs": [
            "*.py",
            "tests/*.py",
            "*/*.md",
            "*/*.ipynb",
            "*/*/*.md",
            "*/*/*.ipynb",
        ],
        f"{pkgname}.subsystems.aero.test.data": ["*.csv"],
        f"{pkgname}.subsystems.prop.test": ["*.csv"],
    },
    entry_points={
        'console_scripts': [
            'aviary=aviary.interface.cmd_entry_points:aviary_cmd',
        ],
        'openmdao_report': [
            'summary=aviary.interface.reports:register_custom_reports',
        ]
    }
)
